Module: HashObject
- Defined in:
- lib/hash_object.rb
Overview
This is a helper module that makes it quite easy to define the Hash -> reified object mapping.
Defined Under Namespace
Modules: ClassMethods Classes: BooleanConverter, ConfigurationError, Element
Class Method Summary collapse
-
.included(base) ⇒ Object
Adds the class methods that are implemented on the included class.
Class Method Details
.included(base) ⇒ Object
Adds the class methods that are implemented on the included class.
6 7 8 9 10 |
# File 'lib/hash_object.rb', line 6 def self.included(base) base.instance_variable_set("@_elements", {}) base.instance_variable_set("@_strict", true) base.extend ClassMethods end |