Exception: HashObject::ConfigurationError
- Inherits:
-
Exception
- Object
- Exception
- HashObject::ConfigurationError
- Defined in:
- lib/hash_object.rb
Overview
An error thrown if a mapping is somehow incorrect.
An error thrown if a mapping is somehow incorrect.